//
//  libraw_cxx_stub.cpp
//  MYRawDecoder
//
//  Created by Dave Thorup on 9/2/20.
//

#include "../internal/libraw_cxx_defs.h"

#include "utils/init_close_utils.cpp"
#include "utils/utils_libraw.cpp"

#include <cassert>

//------------------------------------------------------------------------------
#define DEFINE_FUNC( return_type, func, return_value ) \
    return_type LibRaw::func \
    { assert( false && "Not implemented!" ); \
      return return_value; }

#define DEFINE_VOID_FUNC( func ) \
    void LibRaw::func \
    { assert( false && "Not implemented!" ); }

//------------------------------------------------------------------------------
DEFINE_FUNC( int, open_file(const char *, INT64), LIBRAW_NOT_IMPLEMENTED )

//------------------------------------------------------------------------------
#if defined(_WIN32) || defined(WIN32)
DEFINE_FUNC( int, open_file(const wchar_t *, INT64), LIBRAW_NOT_IMPLEMENTED )
#endif

DEFINE_FUNC( int, open_buffer(void *, size_t), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, open_datastream(LibRaw_abstract_datastream *), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, open_bayer(unsigned char *, unsigned,
                             ushort, ushort,
                             ushort, ushort,
                             ushort, ushort,
                             unsigned char, unsigned char,
                             unsigned, unsigned,
                             unsigned), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, unpack(void), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, unpack_thumb(void), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, thumbOK(INT64), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, adjust_sizes_info_only(void), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, subtract_black(), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, subtract_black_internal(), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, raw2image(), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, raw2image_ex(int), LIBRAW_NOT_IMPLEMENTED )
DEFINE_VOID_FUNC( raw2image_start() )
DEFINE_FUNC( const char*, cameramakeridx2maker(unsigned), nullptr )
DEFINE_FUNC( int, setMakeFromIndex(unsigned), LIBRAW_NOT_IMPLEMENTED )
DEFINE_VOID_FUNC( convertFloatToInt(float, float, float) )
DEFINE_FUNC( const char **, cameraList(), nullptr )
DEFINE_FUNC( int, cameraCount(), 0 )

//------------------------------------------------------------------------------
//  MARK: dcraw emulation
DEFINE_FUNC( int, dcraw_ppm_tiff_writer(const char *), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, dcraw_thumb_writer(const char *x), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, dcraw_process(void), LIBRAW_NOT_IMPLEMENTED )

//------------------------------------------------------------------------------
//  MARK: information calls
DEFINE_FUNC( int, is_sraw(), 0 )
DEFINE_FUNC( int, sraw_midpoint(), 0 )
DEFINE_FUNC( int, is_nikon_sraw(), 0 )
DEFINE_FUNC( int, is_coolscan_nef(), 0 )
DEFINE_FUNC( int, is_jpeg_thumb(), 0 )
DEFINE_FUNC( int, is_floating_point(), 0 )
DEFINE_FUNC( int, have_fpdata(), 0 )

//------------------------------------------------------------------------------
//  MARK: memory writers
DEFINE_FUNC( libraw_processed_image_t *, dcraw_make_mem_image(int *), nullptr )
DEFINE_FUNC( libraw_processed_image_t *, dcraw_make_mem_thumb(int *), nullptr )
DEFINE_VOID_FUNC( dcraw_clear_mem(libraw_processed_image_t *) )

//  MARK: Additional calls for make_mem_image
DEFINE_VOID_FUNC( get_mem_image_format(int *, int *, int *,
                                       int *) const )
DEFINE_FUNC( int, copy_mem_image(void *, int, int), LIBRAW_NOT_IMPLEMENTED )

//------------------------------------------------------------------------------
DEFINE_FUNC( int, fcol(int, int), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( const char *, unpack_function_name(), nullptr )
DEFINE_FUNC( int, get_decoder_info(libraw_decoder_info_t *), LIBRAW_NOT_IMPLEMENTED )

//------------------------------------------------------------------------------
//  MARK: Phase one correction/subtractBL calls
DEFINE_FUNC( int, phase_one_subtract_black(ushort *src, ushort *dest), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, phase_one_correct(), LIBRAW_NOT_IMPLEMENTED )

//------------------------------------------------------------------------------
DEFINE_FUNC( int, set_rawspeed_camerafile(char *filename), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, adobe_coeff(unsigned, const char *, int), LIBRAW_NOT_IMPLEMENTED )
DEFINE_VOID_FUNC( set_dng_host(void *) )
DEFINE_FUNC( void *, memmem(char *, size_t, char *, size_t), nullptr )
DEFINE_FUNC( char *, strcasestr(char *, const char *), nullptr )
DEFINE_FUNC( size_t, strnlen(const char *, size_t), 0 )

//------------------------------------------------------------------------------
DEFINE_VOID_FUNC( cam_xyz_coeff(float _rgb_cam[3][4], double cam_xyz[4][3]) )
DEFINE_VOID_FUNC( phase_one_allocate_tempbuffer() )
DEFINE_VOID_FUNC( phase_one_free_tempbuffer() )
DEFINE_FUNC( int, is_phaseone_compressed(), 0 )
DEFINE_FUNC( int, is_canon_600(), 0 )

//  MARK: Hotspots
DEFINE_VOID_FUNC( copy_fuji_uncropped(unsigned short cblack[4],
                                      unsigned short *dmaxp) )

DEFINE_VOID_FUNC( copy_bayer(unsigned short cblack[4], unsigned short *) )
DEFINE_VOID_FUNC( fuji_rotate() )
DEFINE_VOID_FUNC( convert_to_rgb_loop(float out_cam[3][4]) )
DEFINE_VOID_FUNC( lin_interpolate_loop(int *, int) )
DEFINE_VOID_FUNC( scale_colors_loop(float scale_mul[4]) )

DEFINE_VOID_FUNC( kodak_thumb_loader() )
DEFINE_VOID_FUNC( write_thumb_ppm_tiff(FILE *) )
DEFINE_VOID_FUNC( identify() )
DEFINE_VOID_FUNC( initdata() )
DEFINE_VOID_FUNC( write_ppm_tiff() )
DEFINE_VOID_FUNC( convert_to_rgb() )
DEFINE_VOID_FUNC( remove_zeroes() )
DEFINE_VOID_FUNC( crop_masked_pixels() )
DEFINE_VOID_FUNC( pre_interpolate() )
DEFINE_VOID_FUNC( border_interpolate(int border) )
DEFINE_VOID_FUNC( lin_interpolate() )
DEFINE_VOID_FUNC( vng_interpolate() )
DEFINE_VOID_FUNC( ppg_interpolate() )
DEFINE_VOID_FUNC( cielab(ushort rgb[3], short lab[3]) )
DEFINE_VOID_FUNC( xtrans_interpolate(int) )
DEFINE_VOID_FUNC( ahd_interpolate() )
DEFINE_VOID_FUNC( dht_interpolate() )
DEFINE_VOID_FUNC( aahd_interpolate() )
DEFINE_VOID_FUNC( dcb(int, int) )
DEFINE_VOID_FUNC( fbdd(int) )
DEFINE_VOID_FUNC( exp_bef(float, float) )
DEFINE_VOID_FUNC( bad_pixels(const char *) )
DEFINE_VOID_FUNC( subtract(const char *) )
DEFINE_VOID_FUNC( hat_transform(float *temp, float *base, int st, int size, int sc) )
DEFINE_VOID_FUNC( wavelet_denoise() )
DEFINE_VOID_FUNC( scale_colors() )
DEFINE_VOID_FUNC( median_filter() )
DEFINE_VOID_FUNC( blend_highlights() )
DEFINE_VOID_FUNC( recover_highlights() )
DEFINE_VOID_FUNC( green_matching() )
DEFINE_VOID_FUNC( stretch() )
DEFINE_VOID_FUNC( jpeg_thumb_writer(FILE *tfp, char *thumb, int thumb_length) )
DEFINE_VOID_FUNC( jpeg_thumb() )
DEFINE_VOID_FUNC( ppm_thumb() )
DEFINE_VOID_FUNC( ppm16_thumb() )
DEFINE_VOID_FUNC( layer_thumb() )
DEFINE_VOID_FUNC( rollei_thumb() )
DEFINE_VOID_FUNC( kodak_thumb_load_raw() )

DEFINE_FUNC( unsigned, get4(), 0 )
DEFINE_FUNC( int, flip_index(int, int), 0 )

DEFINE_VOID_FUNC( gamma_curve(double pwr, double ts, int mode, int imax) )
DEFINE_VOID_FUNC( cubic_spline(const int *x_, const int *y_, const int len) )

//  MARK: RawSpeed data
DEFINE_VOID_FUNC( fix_after_rawspeed(int bl) )
DEFINE_FUNC( int, try_rawspeed(), LIBRAW_NOT_IMPLEMENTED )

//  MARK: DNG SDK data
DEFINE_FUNC( int, valid_for_dngsdk(), LIBRAW_NOT_IMPLEMENTED )
DEFINE_FUNC( int, try_dngsdk(), LIBRAW_NOT_IMPLEMENTED )

//------------------------------------------------------------------------------
